package com.kda.data.managers;

import java.io.Serializable;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import com.kda.data.entities.ActividadDescartada;
import com.kda.data.entities.Usuario;
import com.kda.data.utils.PersistenceUtil;

@ManagedBean(name = "actividadDescartadaManager")
@SessionScoped
public class ActividadDescartadaManagerImpl implements Serializable,
		ActividadDescartadaManager {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2018252202837885806L;

	public ActividadDescartadaManagerImpl() {
	}

	/**
	 * Recuperar actividad descartada a través de su id
	 */
	@Override
	public ActividadDescartada recuperaActividadDescartada(int id) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		ActividadDescartada actividadDescartada = null;
		try {
			actividadDescartada = persistenceUtil
					.consulta(
							"select p from ActividadDescartada p where p.id=:id")
					.addParametro("id", id).single(ActividadDescartada.class);
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadDescartada;
	}

	/**
	 * Carga de las actividades descartadas por un usuario.
	 */
	@Override
	public List<ActividadDescartada> cargaActividadesDescartadas(Usuario usuario) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		List<ActividadDescartada> actividadesDescartada = null;
		try {
			actividadesDescartada = persistenceUtil
					.consulta(
							"select p from ActividadDescartada p where p.usuarioId=:usuarioId order by p.fecha")
					.addParametro("usuarioId", usuario.getId())
					.multi(ActividadDescartada.class);
		} catch (Exception e) {
			// No se han podido recuperar las actividades descartadas del
			// usuario
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadesDescartada;
	}

}
