package com.kda.data.managers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.bean.ManagedProperty;
import javax.persistence.TemporalType;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import com.kda.data.entities.Actividad;
import com.kda.data.entities.ActividadConDatos;
import com.kda.data.entities.ActividadDescartada;
import com.kda.data.entities.ActividadSuscrita;
import com.kda.data.entities.Aficion;
import com.kda.data.entities.Ubicacion;
import com.kda.data.entities.Usuario;
import com.kda.data.utils.PersistenceUtil;
import com.kda.utils.Utilidades;

@Service
@Scope(value = "session")
public class ActividadManagerImpl implements ActividadManager, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2018252202837885806L;

	@ManagedProperty(value = "#{aficionManager}")
	AficionManager aficionManager;

	@ManagedProperty(value = "#{ubicacionManager}")
	UbicacionManager ubicacionManager;

	@ManagedProperty(value = "#{comentarioManager}")
	ComentarioManager comentarioManager;

	@ManagedProperty(value = "#{actividadDescartadaManager}")
	ActividadDescartadaManager actividadDescartadaManager;

	@ManagedProperty(value = "#{actividadSuscritaManager}")
	ActividadSuscritaManager actividadSuscritaManager;

	public ActividadManagerImpl() {
	}

	@Override
	public boolean validaDatos(Actividad actividad) {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * Guardado de los datos de la activida
	 */
	@Override
	public boolean guardaDatos(Actividad actividad, Usuario usuario) {
		boolean resultado = false;
		try {
			PersistenceUtil persistenceUtil = new PersistenceUtil();
			persistenceUtil.crearEntityManager(false);
			Actividad aux = this.recuperaActividad(actividad);
			// Asignamos la clave para realizar la actualización
			if (aux != null) {
				actividad.setId(aux.getId());
			}
			persistenceUtil.guardar(actividad);
			// Necesario para recuperar el id del registro insertado
			persistenceUtil.finalizaTransaccion();
			persistenceUtil.crearEntityManager(false);

			// Creación del comentario inicial que sirve para identificar
			// las actividades suscritas

			suscribirseAActividad(actividad, usuario);

			persistenceUtil.finalizaTransaccion();
			resultado = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return resultado;
	}

	/**
	 * Recuperación de una actividad a partir de sus datos
	 */
	@Override
	public Actividad recuperaActividad(Actividad actividad) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		Actividad actividadAux = null;
		try {
			actividadAux = persistenceUtil
					.consulta(
							" select 	p from Actividad p "
									+ " where 	p.usuarioCreadorId=:usuarioId "
									+ " and		p.aficionId=:aficionId "
									+ " and		p.ubicacionId=:ubicacionId"
									+ " and		p.fechaCelebracion=:fecha")
					.addParametro("usuarioId", actividad.getUsuarioCreadorId())
					.addParametro("aficionId", actividad.getAficionId())
					.addParametro("ubicacionId", actividad.getUbicacionId())
					.addParametro("fecha", actividad.getFechaCelebracion())
					.single(Actividad.class);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadAux;
	}

	/**
	 * Recuperación de una actividad a partir de su identificador
	 */
	@Override
	public Actividad recuperaActividad(long actividadId) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		Actividad actividadAux = new Actividad();
		try {
			actividadAux.setId(actividadId);
			actividadAux = persistenceUtil
					.getById(Actividad.class, actividadId);
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadAux;
	}

	/**
	 * Recuperación de una actividad a partir de su identificador
	 */
	@Override
	public ActividadConDatos recuperaActividadConDatos(long actividadId) {
		Actividad actividad = this.recuperaActividad(actividadId);
		ActividadConDatos actividadConDatos = this
				.convertActividadToActividadConDatos(actividad, null);
		return actividadConDatos;
	}

	/**
	 * Recuperación de las actividades de un usuario posteriores a una
	 * determinada fecha
	 */
	@Override
	public List<ActividadConDatos> recuperaActividadesUsuarioPosterioresA(
			Usuario usuario, Date fecha) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		List<ActividadConDatos> actividadesSuscritas = new ArrayList<ActividadConDatos>();
		List<Actividad> actividades = null;
		java.util.Date fecha2 = new java.util.Date();

		try {
			actividades = persistenceUtil
					.consulta(
							" select a from Actividad a, ActividadSuscrita b"
									+ " where a.id = b.actividadId "
									+ " and b.usuarioId = :usuario_id "
									+ " and a.fechaCelebracion >= :fecha "
									+ " order by a.fechaCelebracion asc")
					.addParametro("usuario_id", usuario.getId())
					.addParametro("fecha", fecha2, TemporalType.TIMESTAMP)
					.multi(Actividad.class);
			actividadesSuscritas = this
					.convertActividadesToActividadesSuscritas(actividades,
							persistenceUtil);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadesSuscritas;
	}

	/**
	 * Recuperación de las actividades posteriores a una determinada fecha a las
	 * que un usuario puede querer suscribirse
	 */
	@Override
	public List<ActividadConDatos> recuperaActividadesAvisosUsuarioPosterioresA(
			Usuario usuario, Date fecha) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		List<ActividadConDatos> actividadesAvisos = new ArrayList<ActividadConDatos>();
		List<Actividad> actividades = null;
		List<Long> inAficiones = new ArrayList<Long>();
		List<Long> inUbicaciones = new ArrayList<Long>();
		java.util.Date fecha2 = new java.util.Date();

		try {
			for (Aficion aficion : usuario.getAficiones()) {
				inAficiones.add(aficion.getId());
			}
			for (Ubicacion ubicacion : usuario.getUbicaciones()) {
				inUbicaciones.add(ubicacion.getId());
			}
			actividades = persistenceUtil
					.consulta(
							" select a from Actividad a "
									+ " where  	a.aficionId in :inAficiones "
									+ " and  	a.ubicacionId in :inUbicaciones "
									+ " and		a.usuarioCreadorId <> :usuario_id  "
									+ " and 	a.fechaCelebracion >= :fecha"
									// LA SEGUNDA VEZ QUE SE EJECUTA EL QRY NO
									// RETORNA REGISTROS POR LO QUE LO HACEMOS
									// MANUALMENTE
									// + " and 	a.id not in ( "
									// + " 		select  b.actividadId "
									// + "			from	ComentarioActividad b "
									// +
									// "			where 	b.usuarioCreadorId = :usuario_id "
									// + "			) "
									+ " order by a.fechaCelebracion asc")
					.addParametro("inAficiones", inAficiones)
					.addParametro("inUbicaciones", inUbicaciones)
					.addParametro("usuario_id", usuario.getId())
					.addParametro("fecha", fecha2, TemporalType.TIMESTAMP)
					// .addParametro("usuarioComentario_id", usuario.getId())
					.multi(Actividad.class);
			actividades = eliminaActividadesSuscritas(usuario, actividades);
			actividades = eliminaActividadesDescartadas(usuario, actividades);
			actividadesAvisos = this.convertActividadesToActividadesSuscritas(
					actividades, persistenceUtil);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadesAvisos;
	}

	/**
	 * Eliminamos de la lista de actividades aquellas a las que el usuario ya
	 * está suscrito
	 * 
	 * @param usuario
	 * @param actividades
	 * @return
	 */
	private List<Actividad> eliminaActividadesSuscritas(Usuario usuario,
			List<Actividad> actividades) {
		List<Actividad> aux = new ArrayList<Actividad>();
		List<ActividadSuscrita> actividadesSuscritas = getActividadSuscritaManager()
				.cargaActividadesSuscritas(usuario);
		if (actividadesSuscritas != null) {
			for (Actividad actividad : actividades) {
				boolean hayComentario = false;
				for (ActividadSuscrita actividadSuscritas : actividadesSuscritas) {
					if (actividad.getId() == actividadSuscritas
							.getActividadId()) {
						hayComentario = true;
					}
				}
				if (!hayComentario) {
					aux.add(actividad);
				}
			}
		}
		return aux;
	}

	/**
	 * Eliminamos de la lista de actividades las descartadas por el usuario
	 * 
	 * @param usuario
	 * @param actividades
	 * @return
	 */
	private List<Actividad> eliminaActividadesDescartadas(Usuario usuario,
			List<Actividad> actividades) {
		List<Actividad> aux = new ArrayList<Actividad>();
		List<ActividadDescartada> actividadesDescartada = getActividadDescartadaManager()
				.cargaActividadesDescartadas(usuario);
		if (actividadesDescartada != null) {
			for (Actividad actividad : actividades) {
				boolean hayDescartada = false;
				for (ActividadDescartada actividadDescartada : actividadesDescartada) {
					if (actividad.getId() == actividadDescartada
							.getActividadId()) {
						hayDescartada = true;
					}
				}
				if (!hayDescartada) {
					aux.add(actividad);
				}
			}
		}
		return aux;
	}

	/**
	 * Recuperación de todas las actividades de un usuario.
	 */
	@Override
	public List<ActividadConDatos> recuperaActividadesUsuario(Usuario usuario) {
		Date fechaInicial = new Date(0);
		return recuperaActividadesUsuarioPosterioresA(usuario, fechaInicial);
	}

	public AficionManager getAficionManager() {
		return aficionManager;
	}

	public void setAficionManager(AficionManager aficionManager) {
		this.aficionManager = aficionManager;
	}

	public UbicacionManager getUbicacionManager() {
		return ubicacionManager;
	}

	public void setUbicacionManager(UbicacionManager ubicacionManager) {
		this.ubicacionManager = ubicacionManager;
	}

	public ComentarioManager getComentarioManager() {
		if (comentarioManager == null) {
			this.comentarioManager = new ComentarioManagerImpl();
		}
		return comentarioManager;
	}

	public void setComentarioManager(ComentarioManager comentarioManager) {
		this.comentarioManager = comentarioManager;
	}

	public ActividadDescartadaManager getActividadDescartadaManager() {
		if (actividadDescartadaManager == null) {
			this.actividadDescartadaManager = new ActividadDescartadaManagerImpl();
		}
		return actividadDescartadaManager;
	}

	public void setActividadDescartadaManager(
			ActividadDescartadaManager actividadDescartadaManager) {
		this.actividadDescartadaManager = actividadDescartadaManager;
	}

	public ActividadSuscritaManager getActividadSuscritaManager() {
		if (actividadSuscritaManager == null) {
			this.actividadSuscritaManager = new ActividadSuscritaManagerImpl();
		}
		return actividadSuscritaManager;
	}

	public void setActividadSuscritaManager(
			ActividadSuscritaManager actividadSuscritaManager) {
		this.actividadSuscritaManager = actividadSuscritaManager;
	}

	/**
	 * Se crea amplia la información para las actividades de la lista
	 * 
	 * @param actividades
	 * @param persistenceUtil
	 * @return
	 */
	public List<ActividadConDatos> convertActividadesToActividadesSuscritas(
			List<Actividad> actividades, PersistenceUtil persistenceUtil) {
		List<ActividadConDatos> actividadesSuscritas = new ArrayList<ActividadConDatos>();
		for (Actividad actividad : actividades) {
			ActividadConDatos actividadSuscrita = convertActividadToActividadConDatos(
					actividad, persistenceUtil);
			actividadesSuscritas.add(actividadSuscrita);
		}
		return actividadesSuscritas;
	}

	/**
	 * Se crea amplia la información para las actividades de la lista
	 * 
	 * @param actividades
	 * @param persistenceUtil
	 * @return
	 */
	private ActividadConDatos convertActividadToActividadConDatos(
			Actividad actividad, PersistenceUtil persistenceUtil) {
		if (persistenceUtil == null) {
			persistenceUtil = new PersistenceUtil(true);
		}
		Date hoy = new java.util.Date();
		ActividadConDatos actividadConDatos = new ActividadConDatos(actividad);
		Usuario usuario = persistenceUtil.getById(Usuario.class,
				actividadConDatos.getUsuarioCreadorId());
		if (usuario != null) {
			actividadConDatos.setNombreUsuario(usuario.getNombre());
		}
		Aficion aficion = persistenceUtil.getById(Aficion.class,
				actividadConDatos.getAficionId());
		if (aficion != null) {
			actividadConDatos.setNombreAficion(aficion.getNombre());
		}
		Ubicacion ubicacion = persistenceUtil.getById(Ubicacion.class,
				actividadConDatos.getUbicacionId());
		if (ubicacion != null) {
			actividadConDatos.setNombrePoblacion(ubicacion.getPoblacion());
		}
		long dias = Utilidades.dias(hoy,
				actividadConDatos.getFechaCelebracion());
		actividadConDatos.setDiasFaltan(dias);

		return actividadConDatos;
	}

	/**
	 * Suscripción de un usuario a una actividad
	 */
	@Override
	public boolean suscribirseAActividad(Actividad actividad, Usuario usuario) {
		boolean resultado = false;
		try {
			resultado = actividadSuscritaManager
					.guardaDatos(actividad, usuario);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return resultado;
	}

	/**
	 * Descartar actividad para un determinado usuario
	 */
	@Override
	public boolean descartarseDeActividad(Actividad actividad, Usuario usuario) {
		boolean resultado = false;
		try {
			PersistenceUtil persistenceUtil = new PersistenceUtil();
			persistenceUtil.crearEntityManager(false);
			ActividadDescartada actividadDescartada = new ActividadDescartada();
			actividadDescartada.setActividadId(actividad.getId());
			actividadDescartada.setUsuarioId(usuario.getId());
			actividadDescartada.setFecha(new Date());
			persistenceUtil.guardar(actividadDescartada);
			persistenceUtil.finalizaTransaccion();
			resultado = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return resultado;
	}
}
