package com.kda.data.managers;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import com.kda.data.entities.Actividad;
import com.kda.data.entities.ActividadSuscrita;
import com.kda.data.entities.Usuario;
import com.kda.data.utils.PersistenceUtil;

@ManagedBean(name = "actividadSuscritaManager")
@SessionScoped
public class ActividadSuscritaManagerImpl implements Serializable,
		ActividadSuscritaManager {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2018252202837885806L;

	public ActividadSuscritaManagerImpl() {
	}

	/**
	 * Guardado de los datos de la activida
	 */
	@Override
	public boolean guardaDatos(Actividad actividad, Usuario usuario) {
		boolean resultado = false;
		try {
			PersistenceUtil persistenceUtil = new PersistenceUtil();
			persistenceUtil.crearEntityManager(false);
			ActividadSuscrita actividadSuscrita = new ActividadSuscrita();
			actividadSuscrita.setActividadId(actividad.getId());
			actividadSuscrita.setFecha(new Date());
			actividadSuscrita.setUsuarioId(usuario.getId());
			persistenceUtil.guardar(actividadSuscrita);
			persistenceUtil.finalizaTransaccion();
			resultado = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return resultado;

	}

	/**
	 * Recuperar actividad Suscrita a través de su id
	 */
	@Override
	public ActividadSuscrita recuperaActividadSuscrita(int id) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		ActividadSuscrita actividadSuscrita = null;
		try {
			actividadSuscrita = persistenceUtil
					.consulta(
							"select p from ActividadSuscrita p where p.id=:id")
					.addParametro("id", id).single(ActividadSuscrita.class);
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadSuscrita;
	}

	/**
	 * Carga de las actividades Suscritas por un usuario.
	 */
	@Override
	public List<ActividadSuscrita> cargaActividadesSuscritas(Usuario usuario) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		List<ActividadSuscrita> actividadesSuscrita = null;
		try {
			actividadesSuscrita = persistenceUtil
					.consulta(
							"select p from ActividadSuscrita p where p.usuarioId=:usuarioId order by p.fecha")
					.addParametro("usuarioId", usuario.getId())
					.multi(ActividadSuscrita.class);
		} catch (Exception e) {
			// No se han podido recuperar las actividades Suscritas del
			// usuario
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return actividadesSuscrita;
	}

}
