package com.kda.data.managers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import com.kda.data.entities.Actividad;
import com.kda.data.entities.Aficion;
import com.kda.data.entities.ComentarioActividad;
import com.kda.data.entities.ComentarioActividadConDatos;
import com.kda.data.entities.Usuario;
import com.kda.data.utils.PersistenceUtil;

@ManagedBean(name = "comentarioManager")
@SessionScoped
public class ComentarioManagerImpl implements Serializable, ComentarioManager {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2018252202837885806L;

	public ComentarioManagerImpl() {
	}

	/*
	 * Validamos los datos de un comentario. (No implementado).
	 * 
	 * @see
	 * com.kda.data.managers.ComentarioManager#validaDatos(com.kda.data.entities
	 * .ComentarioActividad)
	 */
	@Override
	public boolean validaDatos(ComentarioActividad comentarioActividad) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * Guarda un comentario en la base de datos.
	 * 
	 * @see
	 * com.kda.data.managers.ComentarioManager#guardaDatos(com.kda.data.entities
	 * .ComentarioActividad)
	 */
	@Override
	public boolean guardaDatos(ComentarioActividad comentarioActividad) {
		boolean resultado = false;
		try {
			PersistenceUtil persistenceUtil = new PersistenceUtil();
			persistenceUtil.crearEntityManager(false);
			ComentarioActividad aux = clone(comentarioActividad);
			persistenceUtil.guardar(aux);
			// Necesario para recuperar el id del registro insertado
			persistenceUtil.finalizaTransaccion();
			resultado = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return resultado;
	}

	/**
	 * Clonamos un objeto de tipo comentarioActividad
	 * 
	 * @param comentarioActividad
	 * @return
	 */
	public static ComentarioActividad clone(
			ComentarioActividad comentarioActividad) {
		ComentarioActividad aux = new ComentarioActividad();
		aux.setActividadId(comentarioActividad.getActividadId());
		aux.setComentario(comentarioActividad.getComentario());
		aux.setComentarioSuscripcion(false);
		aux.setFecha(comentarioActividad.getFecha());
		aux.setUsuarioCreadorId(comentarioActividad.getUsuarioCreadorId());
		return aux;
	}

	/*
	 * Guardamos el comentario inicial de una actividad que sirve para indicar
	 * que un usuario está asociado a la misma.
	 * 
	 * @see
	 * com.kda.data.managers.ComentarioManager#guardaComentarioActividaInicial
	 * (com.kda.data.entities.Actividad, com.kda.data.entities.Usuario,
	 * com.kda.data.utils.PersistenceUtil)
	 */
	@Override
	public void guardaComentarioActividaInicial(Actividad actividad,
			Usuario usuario, PersistenceUtil persistenceUtil) {
		ComentarioActividad comentarioActividad = new ComentarioActividad();
		comentarioActividad.setActividadId(actividad.getId());
		comentarioActividad.setComentario("");
		comentarioActividad.setComentarioSuscripcion(true);
		comentarioActividad.setFecha(actividad.getFechaCelebracion());
		if (usuario == null) {
			comentarioActividad.setUsuarioCreadorId(actividad
					.getUsuarioCreadorId());
		} else {
			comentarioActividad.setUsuarioCreadorId(usuario.getId());
		}
		persistenceUtil.guardar(comentarioActividad);

	}

	/*
	 * Recuperación de todos los comentarios hechos por un usuario
	 * 
	 * @see
	 * com.kda.data.managers.ComentarioManager#getComentariosDelUsuario(com.
	 * kda.data.entities.Usuario)
	 */
	@Override
	public List<ComentarioActividad> getComentariosDelUsuario(Usuario usuario) {
		List<ComentarioActividad> comentarios = null;
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		try {
			comentarios = persistenceUtil
					.consulta(
							" select p from ComentarioActividad p "
									+ " where  	p.usuarioCreadorId = :usuario_id ")
					.addParametro("usuario_id", usuario.getId())
					.multi(ComentarioActividad.class);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			persistenceUtil.finalizaTransaccion();
		}

		return comentarios;
	}

	/*
	 * Recuperación de los comentarios asociados a las actividades a las que el
	 * usuario está suscrito posteriores a la fecha de suscripción a la
	 * actividad.
	 * 
	 * @see com.kda.data.managers.ComentarioManager#
	 * recuperaComentariosActividadesPosterioresA(com.kda.data.entities.Usuario,
	 * java.util.Date)
	 */
	@Override
	public List<ComentarioActividadConDatos> recuperaComentariosActividadesPosterioresA(
			Usuario usuario, Date fecha) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		List<ComentarioActividadConDatos> comentariosConDatos = new ArrayList<ComentarioActividadConDatos>();
		List<ComentarioActividad> comentarios = null;
		java.util.Date fecha2 = new java.util.Date();

		try {
			comentarios = persistenceUtil
					.consulta(
							" select p from ComentarioActividad p, ActividadSuscrita a "
									+ " where p.actividadId = a.actividadId "
									+ " and	a.usuarioId = :usuario_id "
									+ " and p.fecha >= a.fecha "
									+ " order by p.fecha desc ")
					.addParametro("usuario_id", usuario.getId())
					.multi(ComentarioActividad.class);
			comentariosConDatos = this.convertComentariosToComentariosConDatos(
					comentarios, persistenceUtil);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return comentariosConDatos;
	}

	/**
	 * Conversión de la lista de comentarios a una lista de comentarios con
	 * todos los datos asociados
	 * 
	 * @param comentariosActividad
	 * @param persistenceUtil
	 * @return
	 */
	private List<ComentarioActividadConDatos> convertComentariosToComentariosConDatos(
			List<ComentarioActividad> comentariosActividad,
			PersistenceUtil persistenceUtil) {
		List<ComentarioActividadConDatos> comentariosActividadConDatos = new ArrayList<ComentarioActividadConDatos>();
		for (ComentarioActividad comentarioActividad : comentariosActividad) {
			ComentarioActividadConDatos comentarioActividadConDatos = convertComentarioToComentarioConDatos(
					comentarioActividad, persistenceUtil);
			comentariosActividadConDatos.add(comentarioActividadConDatos);
		}
		return comentariosActividadConDatos;
	}

	/**
	 * Conversión de un comentario a un comentario con todos los datos
	 * asociados.
	 * 
	 * @param comentarioActividad
	 * @param persistenceUtil
	 * @return
	 */
	private ComentarioActividadConDatos convertComentarioToComentarioConDatos(
			ComentarioActividad comentarioActividad,
			PersistenceUtil persistenceUtil) {
		Date hoy = new java.util.Date();
		Usuario usuario = null;
		ComentarioActividadConDatos comentarioActividadConDatos = new ComentarioActividadConDatos(
				comentarioActividad);

		Actividad actividad = persistenceUtil.getById(Actividad.class,
				comentarioActividad.getActividadId());
		if (actividad != null) {
			usuario = persistenceUtil.getById(Usuario.class,
					actividad.getUsuarioCreadorId());
			if (usuario != null) {
				comentarioActividadConDatos.setNombreUsuario(usuario
						.getNombre());
			}
			comentarioActividadConDatos.setFechaCelebracionActividad(actividad
					.getFechaCelebracion());
			Aficion aficion = persistenceUtil.getById(Aficion.class,
					actividad.getAficionId());
			if (aficion != null) {
				comentarioActividadConDatos.setNombreAficion(aficion
						.getNombre());
			}
			comentarioActividadConDatos.setDescripcionActividad(actividad
					.getDescripcion());
		}
		return comentarioActividadConDatos;
	}
}
