package com.kda.data.managers;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import com.kda.data.entities.Usuario;
import com.kda.utils.BaseContext;
import com.kda.utils.KdaException;

//@ManagedBean(name = "sesionManager")
//@SessionScoped
@Service
@Scope(value = "session")
public class SesionManagerImpl extends BaseContext implements Serializable,
		SesionManager {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2946901091003412323L;

	private boolean usuarioLogado = false;
	private Usuario usuario = new Usuario();
	private Map<String, String> opcionesPermitidas = new HashMap<String, String>();

	// @ManagedProperty(value = "#{usuarioManager}")
	@Autowired
	UsuarioManager usuarioManager;

	public SesionManagerImpl() {
		rellenaOpcionesPermitidas();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.kda.data.managers.SesionManager#login(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public boolean login(String mail, String password) {
		Usuario aux = null;

		try {
			aux = getUsuarioManager().recuperaUsuario(mail);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (aux != null && password.equals(aux.getPassword())) {
			this.usuario = aux;
			usuarioLogado = true;
		} else {
			usuarioLogado = false;
		}
		rellenaOpcionesPermitidas();
		return this.isUsuarioLogado();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.kda.data.managers.SesionManager#login(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public boolean logout() {
		this.usuario = null;
		usuarioLogado = false;

		rellenaOpcionesPermitidas();
		return this.isUsuarioLogado();
	}

	private void rellenaOpcionesPermitidas() {
		opcionesPermitidas.clear();
		if (this.isUsuarioLogado()) {
			opcionesPermitidas.put("Pizarra", "pizarra");
			opcionesPermitidas.put("Perfil", "perfil");
			opcionesPermitidas.put("Actividades", "actividades");
			opcionesPermitidas.put("Logout", "logout");
		} else {
			opcionesPermitidas.put("Registrarse", "unirse");
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.kda.data.managers.SesionManager#isUsuarioLogado()
	 */
	@Override
	public boolean isUsuarioLogado() {
		return usuarioLogado;
	}

	public void setUsuarioLogado(boolean usuarioLogado) {
		this.usuarioLogado = usuarioLogado;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.kda.data.managers.SesionManager#getUsuario()
	 */
	@Override
	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.kda.data.managers.SesionManager#getOpcionesPermitidas()
	 */
	@Override
	public Map<String, String> getOpcionesPermitidas() {
		return opcionesPermitidas;
	}

	public void setOpcionesPermitidas(Map<String, String> opcionesPermitidas) {
		this.opcionesPermitidas = opcionesPermitidas;
	}

	public UsuarioManager getUsuarioManager() throws KdaException {
		if (this.usuarioManager == null) {
			usuarioManager = (UsuarioManager) BaseContext
					.getBeanFromApplicationContext(UsuarioManager.SERVICIO);
		}
		return usuarioManager;
	}

	public void setUsuarioManager(UsuarioManager usuarioManager) {
		this.usuarioManager = usuarioManager;
	}

}
