package com.kda.data.managers;

import java.io.Serializable;
import java.util.List;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import com.kda.data.entities.Aficion;
import com.kda.data.entities.Ubicacion;
import com.kda.data.entities.Usuario;
import com.kda.data.entities.UsuarioAficion;
import com.kda.data.entities.UsuarioUbicacion;
import com.kda.data.utils.PersistenceUtil;
import com.kda.utils.Constantes;

//@ManagedBean(name = "usuarioManager")
//@SessionScoped
@Service
@Scope(value = "session")
public class UsuarioManagerImpl implements UsuarioManager, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2018252202837885806L;

	public UsuarioManagerImpl() {
	}

	@Override
	public boolean validaDatos(Usuario usuario) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean guardaDatos(Usuario usuario) {
		boolean resultado = false;
		try {
			PersistenceUtil persistenceUtil = new PersistenceUtil();
			persistenceUtil.crearEntityManager(false);
			Usuario aux = this.recuperaUsuario(usuario.getMail());
			// Asignamos la clave para realizar la actualización
			if (aux != null) {
				usuario.setId(aux.getId());
			}
			int tamanoExtracto = usuario.getExtracto().length();
			// Evitamos error de tamaño muy largo
			if (usuario.getExtracto().length() > Constantes.TAMANO_MAXIMO_EXTRACTO) {
				tamanoExtracto = Constantes.TAMANO_MAXIMO_EXTRACTO;
			}
			usuario.setExtracto(usuario.getExtracto().substring(0,
					tamanoExtracto - 1));
			persistenceUtil.guardar(usuario);
			// Necesario para recuperar el id del registro insertado
			persistenceUtil.finalizaTransaccion();
			guardarAficionesUsuario(persistenceUtil, usuario);
			guardarUbicacionesUsuario(persistenceUtil, usuario);
			persistenceUtil.finalizaTransaccion();
			resultado = true;
		} catch (Exception err) {
			err.printStackTrace();
		}
		return resultado;
	}

	private void guardarAficionesUsuario(PersistenceUtil persistenceUtil,
			Usuario usuario) {
		try {
			// Primero borramos las aficiones existentes.
			persistenceUtil.crearEntityManager(false);
			persistenceUtil
					.getEntityManager()
					.createQuery(
							"delete from UsuarioAficion a where a.usuario_id=:id")
					.setParameter("id", usuario.getId()).executeUpdate();
			persistenceUtil.finalizaTransaccion();

			persistenceUtil.crearEntityManager(false);
			for (Aficion aficion : usuario.getAficiones()) {
				UsuarioAficion usuarioAficion = new UsuarioAficion();
				usuarioAficion.setAficion_id(aficion.getId());
				usuarioAficion.setUsuario_id(usuario.getId());
				persistenceUtil.guardar(usuarioAficion);
			}
			persistenceUtil.finalizaTransaccion();
		} catch (Exception e) {
			// TODO: Tratar excepción en la inserción de aficiones del usuario
			e.printStackTrace();
		}
	}

	private void guardarUbicacionesUsuario(PersistenceUtil persistenceUtil,
			Usuario usuario) {
		try {
			persistenceUtil.crearEntityManager(false);
			// Primero borramos las ubicaciones existentes.
			persistenceUtil
					.getEntityManager()
					.createQuery(
							"delete from UsuarioUbicacion a where a.usuario_id=:id")
					.setParameter("id", usuario.getId()).executeUpdate();
			persistenceUtil.finalizaTransaccion();

			persistenceUtil.crearEntityManager(false);
			for (Ubicacion ubicacion : usuario.getUbicaciones()) {
				UsuarioUbicacion usuarioUbicacion = new UsuarioUbicacion();
				usuarioUbicacion.setUbicacion_id(ubicacion.getId());
				usuarioUbicacion.setUsuario_id(usuario.getId());
				persistenceUtil.guardar(usuarioUbicacion);
			}
			persistenceUtil.finalizaTransaccion();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public Usuario recuperaUsuario(String mail) {
		PersistenceUtil persistenceUtil = new PersistenceUtil(true);
		Usuario usuario = null;
		try {
			usuario = persistenceUtil
					.consulta("select p from Usuario p where p.mail=:mail")
					.addParametro("mail", mail).single(Usuario.class);
			if (usuario != null) {
				List<Aficion> aficionesUsuario = persistenceUtil
						.consulta(
								"select p from UsuarioAficion a, Aficion p where a.aficion_id = p.id and a.usuario_id=:id")
						.addParametro("id", usuario.getId())
						.multi(Aficion.class);
				usuario.setAficiones(aficionesUsuario);
				List<Ubicacion> ubicacionUsuario = persistenceUtil
						.consulta(
								"select p from UsuarioUbicacion a, Ubicacion p where a.ubicacion_id = p.id and a.usuario_id=:id")
						.addParametro("id", usuario.getId())
						.multi(Ubicacion.class);
				usuario.setUbicaciones(ubicacionUsuario);
			}
		} finally {
			persistenceUtil.finalizaTransaccion();
		}
		return usuario;
	}

	@Override
	public List<Usuario> cargaUsuarios() {
		// TODO Auto-generated method stub
		return null;
	}

}
