package com.kda.display.controlers;

import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import com.kda.data.entities.ActividadConDatos;
import com.kda.data.entities.ComentarioActividadConDatos;
import com.kda.data.managers.ActividadManager;
import com.kda.data.managers.ComentarioManager;
import com.kda.data.managers.SesionManager;
import com.kda.display.forms.ComentarioActividadForm;
import com.kda.utils.Constantes;

@ManagedBean
@ViewScoped
public class ComentarioActividadControler extends BaseControler implements
		Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5320653523068647752L;
	private static Logger logger = Logger
			.getLogger("ComentarioActividadControler");

	private ComentarioActividadForm comentarioActividadForm;

	@ManagedProperty(value = "#{sesionManager}")
	SesionManager sesionManager;

	@ManagedProperty(value = "#{actividadManager}")
	ActividadManager actividadManager;

	@ManagedProperty(value = "#{comentarioManager}")
	ComentarioManager comentarioManager;

	private String actividadId;

	ComentarioActividadConDatos comentarioActividad = new ComentarioActividadConDatos();

	public ComentarioActividadControler() {

	}

	@PostConstruct
	public void cargaPantalla() {
		FacesContext facesContext = FacesContext.getCurrentInstance();
		this.actividadId = facesContext.getExternalContext()
				.getRequestParameterMap().get("actividadId");
		try {
			long actividadIdLong = Long.valueOf(this.actividadId);
			this.cargaFormDesdeActividad(actividadIdLong);
		} catch (Exception e) {
			logger.log(
					Level.SEVERE,
					"[ComentarioActividadControler.cargaPantalla] Error al intentar recuperar los datos de la actividad");
		}
	}

	/**
	 * Cargamos los datos del formulario de pantalla a partir del Id de la
	 * actividad
	 */
	private void cargaFormDesdeActividad(long id) {
		comentarioActividadForm = new ComentarioActividadForm();
		try {
			ActividadConDatos actividad = actividadManager
					.recuperaActividadConDatos(id);
			comentarioActividad.setActividadId(id);
			comentarioActividad.setNombreAficion(actividad.getNombreAficion());
			comentarioActividad.setFechaCelebracionActividad(actividad
					.getFechaCelebracion());
			comentarioActividad.setDescripcionActividad(actividad
					.getDescripcion());
		} catch (Exception e) {

		}
		comentarioActividadForm.toForm(comentarioActividad, this
				.getSesionManager().getUsuario());
	}

	public String cargaDatos() {
		String resultado = Constantes.PAGINA_COMENTARIO_ACTIVIDAD;
		return resultado;
	}

	public void guardaDatos(ActionEvent actionEvent) {
		ComentarioActividadConDatos comentarioActividad = comentarioActividadForm
				.toObject(null);
		comentarioManager.guardaDatos(comentarioActividad);
	}

	public String guardaDatos() {
		ComentarioActividadConDatos comentarioActividad = comentarioActividadForm
				.toObject(null);
		comentarioManager.guardaDatos(comentarioActividad);
		return Constantes.PAGINA_PIZARRA + "?faces-redirect=true";
	}

	public ComentarioActividadForm getComentarioActividadForm() {
		return comentarioActividadForm;
	}

	public void setComentarioActividadForm(
			ComentarioActividadForm comentarioActividadForm) {
		this.comentarioActividadForm = comentarioActividadForm;
	}

	public SesionManager getSesionManager() {
		return sesionManager;
	}

	public void setSesionManager(SesionManager sesionManager) {
		this.sesionManager = sesionManager;
	}

	public ActividadManager getActividadManager() {
		return actividadManager;
	}

	public void setActividadManager(ActividadManager actividadManager) {
		this.actividadManager = actividadManager;
	}

	public ComentarioManager getComentarioManager() {
		return comentarioManager;
	}

	public void setComentarioManager(ComentarioManager comentarioManager) {
		this.comentarioManager = comentarioManager;
	}

	public String getActividadId() {
		return actividadId;
	}

	public void setActividadId(String actividadId) {
		this.actividadId = actividadId;
	}

	public ComentarioActividadConDatos getComentarioActividad() {
		return comentarioActividad;
	}

	public void setComentarioActividad(
			ComentarioActividadConDatos comentarioActividad) {
		this.comentarioActividad = comentarioActividad;
	}

}
