package com.kda.display.controlers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.event.ActionEvent;
import javax.faces.event.AjaxBehaviorEvent;
import javax.faces.model.SelectItem;

import com.kda.data.entities.AgrupacionAficiones;
import com.kda.data.entities.Provincia;
import com.kda.data.entities.Usuario;
import com.kda.data.managers.AficionManager;
import com.kda.data.managers.AgrupacionAficionesManager;
import com.kda.data.managers.ProvinciasManager;
import com.kda.data.managers.SesionManager;
import com.kda.data.managers.UbicacionManager;
import com.kda.data.managers.UsuarioManager;
import com.kda.display.forms.AficionesForm;
import com.kda.display.forms.AgrupacionAficionesForm;
import com.kda.display.forms.DatosUsuarioForm;
import com.kda.display.forms.ProvinciasForm;
import com.kda.display.forms.UbicacionesForm;
import com.kda.utils.Constantes;

@ManagedBean
@ViewScoped
public class DatosUsuarioControler extends BaseControler implements
		Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5320653523068647752L;
	private Logger logger = Logger.getLogger("DatosUsuarioControler");

	private DatosUsuarioForm datosUsuarioForm;

	@ManagedProperty(value = "#{sesionManager}")
	SesionManager sesionManager;

	@ManagedProperty(value = "#{usuarioManager}")
	UsuarioManager usuarioManager;

	@ManagedProperty(value = "#{agrupacionAficionesManager}")
	AgrupacionAficionesManager agrupacionAficionesManager;

	@ManagedProperty(value = "#{aficionManager}")
	AficionManager aficionManager;

	@ManagedProperty(value = "#{provinciasManager}")
	ProvinciasManager provinciasManager;

	@ManagedProperty(value = "#{ubicacionManager}")
	UbicacionManager ubicacionManager;

	private boolean desdePizarra = false;
	private int provinciaPos;

	public DatosUsuarioControler() {

	}

	@PostConstruct
	public void cargaPantalla() {
		datosUsuarioForm = new DatosUsuarioForm();
		datosUsuarioForm.setAgrupacionAficiones(AgrupacionAficionesForm
				.getAgrupacionAficiones(this.agrupacionAficionesManager
						.cargaTodasLasAgrupaciones()));

		datosUsuarioForm
				.setProvincias(ProvinciasForm
						.getProvincias(this.provinciasManager
								.cargaTodasLasProvincias()));

		if (this.sesionManager.isUsuarioLogado()) {
			this.desdePizarra = true;
			this.datosUsuarioForm.toForm(this.sesionManager.getUsuario());
			datosUsuarioForm.setAficiones(AficionesForm
					.getAficiones(this.aficionManager
							.cargaAficiones(new AgrupacionAficiones(
									this.sesionManager.getUsuario()
											.getAficiones().get(0)
											.getAgrupacion_id()))));
			cargaAficionesSeleccionadas();
			cargaUbicacionesSeleccionadas();
		} else {
			this.datosUsuarioForm.reset();
		}
	}

	public void cargaAficionesSeleccionadas() {
		int pos = 0;
		datosUsuarioForm.setxAficiones(new ArrayList<List<SelectItem>>());
		for (Long agrupacionAficionId : datosUsuarioForm
				.getxAgrupacionAficionId()) {
			if (agrupacionAficionId != -1) {
				datosUsuarioForm.getxAficiones().add(
						pos,
						AficionesForm.getAficiones(this.aficionManager
								.cargaAficiones(new AgrupacionAficiones(
										agrupacionAficionId))));
			} else {
				datosUsuarioForm.getxAficiones().add(pos,
						new ArrayList<SelectItem>());
			}
			pos++;

		}
	}

	public void cargaUbicacionesSeleccionadas() {
		int pos = 0;
		datosUsuarioForm.setUbicaciones(new ArrayList<List<SelectItem>>());
		for (Long provinciaId : datosUsuarioForm.getProvinciaId()) {
			if (provinciaId != -1) {
				datosUsuarioForm.getUbicaciones().add(
						pos,
						UbicacionesForm.getUbicaciones(this.ubicacionManager
								.cargaUbicacion(new Provincia(provinciaId))));
			} else {
				datosUsuarioForm.getUbicaciones().add(pos,
						new ArrayList<SelectItem>());
			}
			pos++;

		}
	}

	public void cambiaAgrupacionAficion(AjaxBehaviorEvent e) {
		try {
			String pos = e.getComponent().getId()
					.substring(e.getComponent().getId().length() - 1);
			int posi = Integer.valueOf(pos);
			if (datosUsuarioForm.getxAficiones().get(posi) == null) {
				datosUsuarioForm.getxAficiones().set(posi,
						new ArrayList<SelectItem>());
			}
			datosUsuarioForm.getxAficiones().get(posi).clear();
			datosUsuarioForm
					.getxAficiones()
					.get(posi)
					.addAll(AficionesForm.getAficiones(this.aficionManager
							.cargaAficiones(new AgrupacionAficiones(
									this.datosUsuarioForm
											.getxAgrupacionAficionId()
											.get(posi)))));
		} catch (Exception er) {
			logger.log(
					Level.SEVERE,
					"[DatosUsuarioControer.cambioProvincia] Error al realizar el cambio de provincia.");
		}
	}

	public void cambiaProvincia(AjaxBehaviorEvent e) {
		try {
			String pos = e.getComponent().getId()
					.substring(e.getComponent().getId().length() - 1);
			int posi = Integer.valueOf(pos);
			if (datosUsuarioForm.getUbicaciones().get(posi) == null) {
				datosUsuarioForm.getUbicaciones().set(posi,
						new ArrayList<SelectItem>());
			}
			datosUsuarioForm.getUbicaciones().get(posi).clear();
			datosUsuarioForm
					.getUbicaciones()
					.get(posi)
					.addAll(UbicacionesForm.getUbicaciones(this.ubicacionManager
							.cargaUbicacion(new Provincia(this.datosUsuarioForm
									.getProvinciaId().get(posi)))));
		} catch (Exception er) {
			logger.log(
					Level.SEVERE,
					"[DatosUsuarioControer.cambioProvincia] Error al realizar el cambio de provincia.");
		}
	}

	public String cargaDatos() {
		String resultado = Constantes.PAGINA_DATOS_USUARIO;
		return resultado;
	}

	public void guardaDatos(ActionEvent actionEvent) {
		Usuario usuario = datosUsuarioForm.toObject(null);
		usuarioManager.guardaDatos(usuario);
	}

	public String guardaDatos() {
		Usuario usuario = datosUsuarioForm.toObject(null);
		boolean resultado = usuarioManager.guardaDatos(usuario);
		if (resultado) {
			sesionManager.login(usuario.getMail(), usuario.getPassword());
			return Constantes.PAGINA_PIZARRA;
		} else {
			return Constantes.PAGINA_NO_REDIRECT;
		}
	}

	public DatosUsuarioForm getDatosUsuarioForm() {
		return datosUsuarioForm;
	}

	public void setDatosUsuarioForm(DatosUsuarioForm datosUsuarioForm) {
		this.datosUsuarioForm = datosUsuarioForm;
	}

	public void setUsuarioManager(UsuarioManager usuarioManager) {
		this.usuarioManager = usuarioManager;
	}

	public SesionManager getSesionManager() {
		return sesionManager;
	}

	public void setSesionManager(SesionManager sesionManager) {
		this.sesionManager = sesionManager;
	}

	public AgrupacionAficionesManager getAgrupacionAficionesManager() {
		return agrupacionAficionesManager;
	}

	public void setAgrupacionAficionesManager(
			AgrupacionAficionesManager agrupacionAficionesManager) {
		this.agrupacionAficionesManager = agrupacionAficionesManager;
	}

	public AficionManager getAficionManager() {
		return aficionManager;
	}

	public void setAficionManager(AficionManager aficionManager) {
		this.aficionManager = aficionManager;
	}

	public UsuarioManager getUsuarioManager() {
		return usuarioManager;
	}

	public ProvinciasManager getProvinciasManager() {
		return provinciasManager;
	}

	public UbicacionManager getUbicacionManager() {
		return ubicacionManager;
	}

	public void setProvinciasManager(ProvinciasManager provinciasManager) {
		this.provinciasManager = provinciasManager;
	}

	public void setUbicacionManager(UbicacionManager ubicacionManager) {
		this.ubicacionManager = ubicacionManager;
	}

	public boolean isDesdePizarra() {
		return desdePizarra;
	}

	public void setDesdePizarra(boolean desdePizarra) {
		this.desdePizarra = desdePizarra;
	}

	public int getProvinciaPos() {
		return provinciaPos;
	}

	public void setProvinciaPos(int provinciaPos) {
		this.provinciaPos = provinciaPos;
	}
}
