package com.kda.display.controlers;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.event.ActionEvent;

import com.kda.data.entities.Actividad;
import com.kda.data.entities.ActividadConDatos;
import com.kda.data.entities.AgrupacionAficiones;
import com.kda.data.entities.ComentarioActividadConDatos;
import com.kda.data.entities.Usuario;
import com.kda.data.managers.ActividadManager;
import com.kda.data.managers.AficionManager;
import com.kda.data.managers.AgrupacionAficionesManager;
import com.kda.data.managers.ComentarioManager;
import com.kda.data.managers.ProvinciasManager;
import com.kda.data.managers.SesionManager;
import com.kda.data.managers.UbicacionManager;
import com.kda.data.managers.UsuarioManager;
import com.kda.display.forms.AficionesForm;
import com.kda.display.forms.DatosUsuarioForm;
import com.kda.display.forms.PizarraForm;
import com.kda.utils.Constantes;

@ManagedBean
@ViewScoped
public class PizarraControler extends BaseControler implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5320653523068647752L;

	private PizarraForm pizarraForm = new PizarraForm();

	@ManagedProperty(value = "#{sesionManager}")
	SesionManager sesionManager;

	@ManagedProperty(value = "#{usuarioManager}")
	UsuarioManager usuarioManager;

	@ManagedProperty(value = "#{agrupacionAficionesManager}")
	AgrupacionAficionesManager agrupacionAficionesManager;

	@ManagedProperty(value = "#{aficionManager}")
	AficionManager aficionManager;

	@ManagedProperty(value = "#{provinciasManager}")
	ProvinciasManager provinciasManager;

	@ManagedProperty(value = "#{ubicacionManager}")
	UbicacionManager ubicacionManager;

	@ManagedProperty(value = "#{actividadManager}")
	ActividadManager actividadManager;

	@ManagedProperty(value = "#{comentarioManager}")
	ComentarioManager comentarioManager;

	private String actividadId;

	public PizarraControler() {

	}

	@PostConstruct
	public void cargaPantalla() {
		try {
			cargarDatosPerfil();
			cargarDatosActividadesSuscritas();
			cargarDatosActividadesAvisos();
			cargarComentariosActividades();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void cargarDatosPerfil() {
		datosUsuarioForm = new DatosUsuarioForm();

		this.datosUsuarioForm.toForm(this.sesionManager.getUsuario());
		datosUsuarioForm.setAficiones(AficionesForm
				.getAficiones(this.aficionManager
						.cargaAficiones(new AgrupacionAficiones(
								this.sesionManager.getUsuario().getAficiones()
										.get(0).getAgrupacion_id()))));
	}

	public void cargarDatosActividadesSuscritas() {
		Usuario usuario = this.sesionManager.getUsuario();
		List<ActividadConDatos> actividadesSuscritas = actividadManager
				.recuperaActividadesUsuarioPosterioresA(usuario,
						new java.util.Date());
		if (actividadesSuscritas != null) {
			this.pizarraForm.setActividadesSuscritas(actividadesSuscritas);
		}
	}

	public void cargarDatosActividadesAvisos() {
		Usuario usuario = this.sesionManager.getUsuario();
		List<ActividadConDatos> actividadesAvisos = actividadManager
				.recuperaActividadesAvisosUsuarioPosterioresA(usuario,
						new java.util.Date());
		if (actividadesAvisos != null) {
			this.pizarraForm.setActividadesAvisos(actividadesAvisos);
		}
	}

	public void cargarComentariosActividades() {
		Usuario usuario = this.sesionManager.getUsuario();
		List<ComentarioActividadConDatos> comentarioActividadConDatos = comentarioManager
				.recuperaComentariosActividadesPosterioresA(usuario, new Date());
		if (comentarioActividadConDatos != null) {
			this.pizarraForm
					.setActividadesComentarios(comentarioActividadConDatos);
		}
	}

	public void editarDatos(ActionEvent actionEvent) {
		Usuario usuario = datosUsuarioForm.toObject(null);
		usuarioManager.guardaDatos(usuario);
	}

	public String editarDatos() {
		return Constantes.PAGINA_DATOS_USUARIO;
	}

	public String nuevaActividad() {
		return Constantes.PAGINA_DATOS_ACTIVIDAD + "?faces-redirect=true";
	}

	public String verActividad() {
		System.out.println("verActividad "
				+ Long.toString(this.getPizarraForm()
						.getActividadSeleccionada()));
		return Constantes.PAGINA_DATOS_ACTIVIDAD;
		// + "?faces-redirect=true&includeViewParams=true";
	}

	public String suscribirActividad() {
		System.out.println(this.getActividadId());
		try {
			long actividadIdLong = Long.valueOf(this.getActividadId());
			Actividad actividad = actividadManager
					.recuperaActividad(actividadIdLong);
			actividadManager.suscribirseAActividad(actividad,
					this.sesionManager.getUsuario());
		} catch (Exception e) {
			// No se ha podido suscribir a la actividad
		}
		return Constantes.PAGINA_PIZARRA;
	}

	public String descartarActividad() {
		System.out.println(this.getActividadId());
		try {
			long actividadIdLong = Long.valueOf(this.getActividadId());
			Actividad actividad = actividadManager
					.recuperaActividad(actividadIdLong);
			actividadManager.descartarseDeActividad(actividad,
					this.sesionManager.getUsuario());
		} catch (Exception e) {
			// No se ha podido suscribir a la actividad
		}
		return Constantes.PAGINA_PIZARRA;
	}

	public String cargaDatos() {
		String resultado = Constantes.PAGINA_PIZARRA;
		return resultado;
	}

	public PizarraForm getPizarraForm() {
		return pizarraForm;
	}

	public void setPizarraForm(PizarraForm pizarraForm) {
		this.pizarraForm = pizarraForm;
	}

	public DatosUsuarioForm getDatosUsuarioForm() {
		return datosUsuarioForm;
	}

	public void setDatosUsuarioForm(DatosUsuarioForm datosUsuarioForm) {
		this.datosUsuarioForm = datosUsuarioForm;
	}

	public void setUsuarioManager(UsuarioManager usuarioManager) {
		this.usuarioManager = usuarioManager;
	}

	public SesionManager getSesionManager() {
		return sesionManager;
	}

	public void setSesionManager(SesionManager sesionManager) {
		this.sesionManager = sesionManager;
	}

	public AgrupacionAficionesManager getAgrupacionAficionesManager() {
		return agrupacionAficionesManager;
	}

	public void setAgrupacionAficionesManager(
			AgrupacionAficionesManager agrupacionAficionesManager) {
		this.agrupacionAficionesManager = agrupacionAficionesManager;
	}

	public AficionManager getAficionManager() {
		return aficionManager;
	}

	public void setAficionManager(AficionManager aficionManager) {
		this.aficionManager = aficionManager;
	}

	public UsuarioManager getUsuarioManager() {
		return usuarioManager;
	}

	public ProvinciasManager getProvinciasManager() {
		return provinciasManager;
	}

	public UbicacionManager getUbicacionManager() {
		return ubicacionManager;
	}

	public void setProvinciasManager(ProvinciasManager provinciasManager) {
		this.provinciasManager = provinciasManager;
	}

	public void setUbicacionManager(UbicacionManager ubicacionManager) {
		this.ubicacionManager = ubicacionManager;
	}

	public ActividadManager getActividadManager() {
		return actividadManager;
	}

	public void setActividadManager(ActividadManager actividadManager) {
		this.actividadManager = actividadManager;
	}

	public ComentarioManager getComentarioManager() {
		return comentarioManager;
	}

	public void setComentarioManager(ComentarioManager comentarioManager) {
		this.comentarioManager = comentarioManager;
	}

	public String getActividadId() {
		return actividadId;
	}

	public void setActividadId(String actividadId) {
		this.actividadId = actividadId;
	}
}
