package com.kda.display.forms;

import java.awt.Image;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import com.kda.data.entities.Aficion;
import com.kda.data.entities.Ubicacion;
import com.kda.data.entities.Usuario;
import com.kda.utils.Constantes;

public class DatosUsuarioForm implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6487640530577644733L;

	private AgrupacionAficionesForm agrupacionesAficionesForm;

	private String mail;
	private String password;
	private String nombre;
	private String apellido1;
	private String apellido2;
	private String telefono;
	private String movil;
	private String extracto;

	private Long agrupacionAficionId;
	private Long aficionId;

	private List<Long> xAgrupacionAficionId;
	private List<Long> xAficionId;

	private List<Long> provinciaId;
	private List<Long> ubicacionId;

	private List<SelectItem> agrupacionAficiones;
	private List<SelectItem> aficiones;
	private List<List<SelectItem>> xAficiones;
	private List<SelectItem> provincias;
	private List<List<SelectItem>> ubicaciones;

	private Image foto;

	public Usuario toObject(Usuario usuario) {
		if (usuario == null) {
			usuario = new Usuario();
		}
		usuario.setMail(getMail());
		usuario.setPassword(getPassword());
		usuario.setNombre(getNombre());
		usuario.setApellido1(getApellido1());
		usuario.setApellido2(getApellido2());
		usuario.setTelefono(getTelefono());
		usuario.setMovil(getMovil());
		usuario.setExtracto(getExtracto());
		usuario.setAficiones(cargaListaxAficiones());
		usuario.setUbicaciones(cargaListaUbicaciones());
		return usuario;
	}

	private List<Aficion> cargaListaxAficiones() {
		List<Aficion> aficionesUsuario = new ArrayList<Aficion>();
		for (Long aficionId : getxAficionId()) {
			if (aficionId != 0) {
				Aficion aficion = new Aficion();
				aficion.setId(aficionId);
				aficionesUsuario.add(aficion);
			}
		}
		return aficionesUsuario;
	}

	private List<Ubicacion> cargaListaUbicaciones() {
		List<Ubicacion> ubicacionesUsuario = new ArrayList<Ubicacion>();
		for (Long ubicacionId : getUbicacionId()) {
			if (ubicacionId != 0) {
				Ubicacion ubicacion = new Ubicacion();
				ubicacion.setId(ubicacionId);
				ubicacionesUsuario.add(ubicacion);
			}
		}
		return ubicacionesUsuario;
	}

	public void toForm(Usuario usuario) {
		setMail(usuario.getMail());
		setPassword(usuario.getPassword());
		setNombre(usuario.getNombre());
		setApellido1(usuario.getApellido1());
		setApellido2(usuario.getApellido2());
		setTelefono(usuario.getTelefono());
		setMovil(usuario.getMovil());
		setExtracto(usuario.getExtracto());

		setAgrupacionAficionId(usuario.getAficiones().get(0).getAgrupacion_id());
		setAficionId(usuario.getAficiones().get(0).getId());

		setxAgrupacionAficionId(getAgrupacionAficionIds(usuario.getAficiones()));
		setxAficionId(getAficionIds(usuario.getAficiones()));

		setProvinciaId(getProvinciasIds(usuario.getUbicaciones()));
		setUbicacionId(getUbicacionesIds(usuario.getUbicaciones()));
	}

	private List<Long> getAgrupacionAficionIds(List<Aficion> aficiones) {
		List<Long> xAgrupacionIds = new ArrayList<Long>();
		for (Aficion aficion : aficiones) {
			Long aficionId = aficion.getAgrupacion_id();
			xAgrupacionIds.add(aficionId);
		}
		for (int i = xAgrupacionIds.size(); i < Constantes.LIMITE_ITEMS_AFICIONES; i++) {
			xAgrupacionIds.add(new Long(-1));
		}
		return xAgrupacionIds;
	}

	private List<Long> getAficionIds(List<Aficion> aficiones) {
		List<Long> aficionesIds = new ArrayList<Long>();
		for (Aficion aficion : aficiones) {
			Long aficionId = aficion.getId();
			aficionesIds.add(aficionId);
		}
		for (int i = aficionesIds.size(); i < Constantes.LIMITE_ITEMS_AFICIONES; i++) {
			aficionesIds.add(new Long(-1));
		}
		return aficionesIds;
	}

	private List<List<SelectItem>> resetAficiones() {
		List<List<SelectItem>> aficiones = new ArrayList<List<SelectItem>>();
		for (int i = 0; i < Constantes.LIMITE_ITEMS_AFICIONES; i++) {
			aficiones.add(null);
		}
		return aficiones;
	}

	private List<Long> getProvinciasIds(List<Ubicacion> ubicaciones) {
		List<Long> provinciasIds = new ArrayList<Long>();
		for (Ubicacion ubicacion : ubicaciones) {
			Long provinciaId = ubicacion.getIdProvincia();
			provinciasIds.add(provinciaId);
		}
		for (int i = provinciasIds.size(); i < Constantes.LIMITE_ITEMS_UBICACIONES; i++) {
			provinciasIds.add(new Long(-1));
		}
		return provinciasIds;
	}

	private List<Long> getUbicacionesIds(List<Ubicacion> ubicaciones) {
		List<Long> ubicacionesIds = new ArrayList<Long>();
		for (Ubicacion ubicacion : ubicaciones) {
			Long ubicacionId = ubicacion.getId();
			ubicacionesIds.add(ubicacionId);
		}
		for (int i = ubicacionesIds.size(); i < Constantes.LIMITE_ITEMS_UBICACIONES; i++) {
			ubicacionesIds.add(new Long(-1));
		}
		return ubicacionesIds;
	}

	private List<List<SelectItem>> resetUbicaciones() {
		List<List<SelectItem>> ubicaciones = new ArrayList<List<SelectItem>>();
		for (int i = 0; i < Constantes.LIMITE_ITEMS_UBICACIONES; i++) {
			ubicaciones.add(null);
		}
		return ubicaciones;
	}

	public void reset() {
		setMail("");
		setPassword("");
		setNombre("");
		setApellido1("");
		setApellido2("");
		setTelefono("");
		setMovil("");
		setExtracto("");

		setxAgrupacionAficionId(getAgrupacionAficionIds(new ArrayList<Aficion>()));
		setxAficionId(getAficionIds(new ArrayList<Aficion>()));
		setxAficiones(resetAficiones());

		setProvinciaId(getProvinciasIds(new ArrayList<Ubicacion>()));
		setUbicacionId(getUbicacionesIds(new ArrayList<Ubicacion>()));
		setUbicaciones(resetUbicaciones());
	}

	public String getMail() {
		return mail;
	}

	public void setMail(String mail) {
		this.mail = mail;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getApellido1() {
		return apellido1;
	}

	public void setApellido1(String apellido1) {
		this.apellido1 = apellido1;
	}

	public String getApellido2() {
		return apellido2;
	}

	public void setApellido2(String apellido2) {
		this.apellido2 = apellido2;
	}

	public String getTelefono() {
		return telefono;
	}

	public void setTelefono(String telefono) {
		this.telefono = telefono;
	}

	public String getMovil() {
		return movil;
	}

	public void setMovil(String movil) {
		this.movil = movil;
	}

	public String getExtracto() {
		return extracto;
	}

	public void setExtracto(String extracto) {
		this.extracto = extracto;
	}

	public List<SelectItem> getAficiones() {
		return aficiones;
	}

	public void setAficiones(List<SelectItem> aficiones) {
		this.aficiones = aficiones;
	}

	public List<List<SelectItem>> getUbicaciones() {
		return ubicaciones;
	}

	public void setUbicaciones(List<List<SelectItem>> ubicaciones) {
		this.ubicaciones = ubicaciones;
	}

	public Image getFoto() {
		return foto;
	}

	public void setFoto(Image foto) {
		this.foto = foto;
	}

	public Long getAgrupacionAficionId() {
		return agrupacionAficionId;
	}

	public void setAgrupacionAficionId(Long agrupacionAficionId) {
		this.agrupacionAficionId = agrupacionAficionId;
	}

	public List<SelectItem> getAgrupacionAficiones() {
		return agrupacionAficiones;
	}

	public void setAgrupacionAficiones(List<SelectItem> agrupacionAficiones) {
		this.agrupacionAficiones = agrupacionAficiones;
	}

	public Long getAficionId() {
		return aficionId;
	}

	public void setAficionId(Long aficionId) {
		this.aficionId = aficionId;
	}

	public List<Long> getProvinciaId() {
		return provinciaId;
	}

	public List<Long> getUbicacionId() {
		return ubicacionId;
	}

	public void setProvinciaId(List<Long> provinciaId) {
		this.provinciaId = provinciaId;
	}

	public void setUbicacionId(List<Long> ubicacionId) {
		this.ubicacionId = ubicacionId;
	}

	public List<SelectItem> getProvincias() {
		return provincias;
	}

	public void setProvincias(List<SelectItem> provincias) {
		this.provincias = provincias;
	}

	public List<Long> getxAgrupacionAficionId() {
		return xAgrupacionAficionId;
	}

	public void setxAgrupacionAficionId(List<Long> xAgrupacionAficionId) {
		this.xAgrupacionAficionId = xAgrupacionAficionId;
	}

	public List<Long> getxAficionId() {
		return xAficionId;
	}

	public void setxAficionId(List<Long> xAficionId) {
		this.xAficionId = xAficionId;
	}

	public List<List<SelectItem>> getxAficiones() {
		return xAficiones;
	}

	public void setxAficiones(List<List<SelectItem>> xAficiones) {
		this.xAficiones = xAficiones;
	}

}
