CREATE DATABASE kda;

CREATE USER kda IDENTIFIED BY 'kdapassword';

GRANT ALL ON kda.* TO 'kda';

CREATE TABLE kda.AGRUPACION_AFICIONES
	(
		id			int UNSIGNED NOT NULL auto_increment, 
		nombre		VARCHAR(50) NOT NULL,
		PRIMARY KEY (id)
	);

CREATE TABLE kda.AFICION
	(
		id 			int UNSIGNED NOT NULL auto_increment,
		agrupacion_id 	int UNSIGNED,
		nombre 		VARCHAR(50) NOT NULL,
		descripcion		VARCHAR(100) DEFAULT NULL,
		PRIMARY KEY (id),
		FOREIGN KEY (agrupacion_id) REFERENCES AGRUPACION_AFICIONES(id)
	);

/*	Tablas creadas en creacionPoblaciones.sql */
CREATE TABLE kda.PROVINCIA ( 
	id int unsigned NOT NULL auto_increment, 
	provincia varchar(50) NOT NULL, 
	provinciaseo varchar(50) NOT NULL, 
	provincia3 char(3) default NULL, 
	PRIMARY KEY (id), 
	UNIQUE KEY provinciaseo (provinciaseo) 
);

CREATE TABLE kda.UBICACION ( 
	id int unsigned NOT NULL auto_increment, 
	idprovincia int unsigned NOT NULL, 
	poblacion varchar(150) NOT NULL, 
	poblacionseo varchar(150) default NULL, 
	postal int(5) unsigned zerofill default NULL, 
	latitud decimal(9,6) default NULL, 
	longitud decimal(9,6) default NULL, 
	PRIMARY KEY (id), 
	UNIQUE KEY poblacionseo (poblacionseo), 
	UNIQUE KEY lugar (latitud,longitud), 
	KEY idprovincia (idprovincia) 
);

CREATE TABLE kda.USUARIO
(
	id 			int UNSIGNED NOT NULL auto_increment,
	mail			VARCHAR(50) NOT NULL, 
	password		VARCHAR(50) NOT NULL,  
	nombre		VARCHAR(50) NOT NULL, 
	apellido1		VARCHAR(50) DEFAULT NULL, 
	apellido2		VARCHAR(50) DEFAULT NULL, 
	telefono		VARCHAR(20) DEFAULT NULL, 
	movil			VARCHAR(20) DEFAULT NULL, 
	extracto		VARCHAR(500) NOT NULL,
	PRIMARY KEY (id)
);

CREATE TABLE kda.ACTIVIDAD
(
	id 				int 	UNSIGNED NOT NULL auto_increment, 
	usuarioCreador	int	UNSIGNED,
	aficion_id		int	UNSIGNED,
	ubicacion_id	int(10)	UNSIGNED,
	fechaCelebracion	timeStamp NOT NULL,
	quedarEn		VARCHAR(200) NOT NULL,
	descripcion		VARCHAR(500) NOT NULL,
	PRIMARY KEY (id),
	FOREIGN		KEY (usuarioCreador) 	REFERENCES USUARIO(id),
	FOREIGN		KEY (aficion_id) 		REFERENCES AFICION(id),
	FOREIGN		KEY (ubicacion_id) 		REFERENCES UBICACION(id)
);

CREATE TABLE kda.COMENTARIO_ACTIVIDAD
(
	id 				int UNSIGNED NOT NULL auto_increment, 
	usuario_id		int	UNSIGNED, 
	actividad_id	int	UNSIGNED,  
	fecha			timestamp NOT NULL, 
	comentario		VARCHAR(500) NOT NULL, 
	comentarioSuscripcion	int UNSIGNED,
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id),
	FOREIGN		KEY (actividad_id) REFERENCES ACTIVIDAD(id)
);

CREATE TABLE kda.ACTIVIDAD_SUSCRITA
(
	id 				int UNSIGNED NOT NULL auto_increment, 
	usuario_id		int	UNSIGNED, 
	actividad_id	int	UNSIGNED,  
	fecha			timestamp NOT NULL, 
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id),
	FOREIGN		KEY (actividad_id) REFERENCES ACTIVIDAD(id)
);

CREATE TABLE kda.ACTIVIDAD_DESCARTADA
(
	id 				int UNSIGNED NOT NULL auto_increment, 
	usuario_id		int	UNSIGNED, 
	actividad_id	int	UNSIGNED,  
	fecha			timestamp NOT NULL, 
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id),
	FOREIGN		KEY (actividad_id) REFERENCES ACTIVIDAD(id)
);

CREATE TABLE kda.USUARIO_FOTO
(
	id 				int UNSIGNED NOT NULL auto_increment, 
	usuario_id		int	UNSIGNED,
	foto			LONGBLOB NOT NULL, 
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id)
);

CREATE TABLE kda.USUARIO_AFICION
(
	id 			int 	UNSIGNED NOT NULL auto_increment, 
	usuario_id		int 	UNSIGNED,
	aficion_id		int 	UNSIGNED,
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id),
	FOREIGN		KEY (aficion_id) REFERENCES AFICION(id)
);

CREATE TABLE kda.USUARIO_UBICACION
(
	id 			int 	UNSIGNED NOT NULL auto_increment, 
	usuario_id		int 	UNSIGNED,
	ubicacion_id	int 	UNSIGNED,
	PRIMARY KEY (id),
	FOREIGN		KEY (usuario_id) REFERENCES USUARIO(id),
	FOREIGN		KEY (ubicacion_id) REFERENCES UBICACION(id)
);

/* 
	Tabla necesaria para las secuencias automáticas de la aplicación
	Se usa por OpenJPA
*/
CREATE TABLE kda.OPENJPA_SEQUENCE_TABLE
	(
		id 				tinyint(4) NOT NULL, 
		SEQUENCE_VALUE	bigint(20) default NULL, 
		PRIMARY KEY (id)
	)
